import logging
import os
import warnings
from typing import Optional

import orjson
from beartype import beartype
from beartype.roar import BeartypeDecorHintPep585DeprecationWarning

from picsellia.colors import Colors
from picsellia.decorators import exception_handler
from picsellia.sdk.connexion import Connexion
from picsellia.sdk.dao import Dao
from picsellia.types.schemas import ScanFileSchema
from picsellia.utils import filter_payload

logger = logging.getLogger("picsellia")
warnings.filterwarnings("ignore", category=BeartypeDecorHintPep585DeprecationWarning)


class ScanFile(Dao):
    def __init__(self, connexion: Connexion, data: dict):
        Dao.__init__(self, connexion, data)

    def __str__(self):
        return f"{Colors.GREEN}ScanFile {self.name}{Colors.ENDC} (id: {self.id})"

    @property
    def name(self) -> str:
        """Name of this (ScanFile)"""
        return self._name

    @property
    def object_name(self) -> str:
        """Object name of this (ScanFile)"""
        return self._object_name

    @property
    def large(self) -> bool:
        """If True, this (ScanFile) is considered large"""
        return self._large

    @exception_handler
    @beartype
    def refresh(self, data: dict) -> ScanFileSchema:
        schema = ScanFileSchema(**data)
        self._name = schema.name
        self._object_name = schema.object_name
        self._large = schema.large
        return schema

    @exception_handler
    @beartype
    def sync(self) -> dict:
        r = self.connexion.get(f"/sdk/scan/file/{self.id}").json()
        self.refresh(r)
        return r

    @exception_handler
    @beartype
    def update(
        self,
        name: Optional[str] = None,
        object_name: Optional[str] = None,
        large: Optional[bool] = None,
    ) -> None:
        """Update this scan file.

        Examples:
            ```python
            script.update(object_name="another-path-to-script")
            ```
        """
        payload = {
            "name": name,
            "object_name": object_name,
            "large": large,
        }
        filtered_payload = filter_payload(payload)
        r = self.connexion.patch(
            f"/sdk/scan/file/{self.id}", data=orjson.dumps(filtered_payload)
        ).json()
        self.refresh(r)

    @exception_handler
    @beartype
    def delete(self) -> None:
        """Delete this scan file

        Examples:
            ```python
            script.delete()
            ```
        """
        self.connexion.delete(f"/sdk/scan/file/{self.id}")

    @exception_handler
    @beartype
    def download(self, target_path: str = "./", force_replace: bool = False) -> None:
        """Download an experiment's artifact to a given target_path.

        Examples:
            ```python
            script.download("myDir")
            file_list = os.path.listdir("myDir")
            print(file_list)
            >>> ["saved_model.zip"]
            ```
        Arguments:
            target_path (str, optional): Path to download the file to, default to cwd. Defaults to './'.
            force_replace: (bool, optional): Replace an existing file if exists. Defaults to False.
        """
        self.sync()
        path = os.path.join(target_path, self.name)
        if self.connexion.download_file(
            self.object_name, path, self.large, force_replace=force_replace
        ):
            logger.info(f"{self.name} downloaded successfully")
        else:
            logger.error(f"Did not download file '{self.name}'")



import logging
from functools import partial
from typing import List, Optional, Union

import orjson
from beartype import beartype

import picsellia.utils as utils
from picsellia.colors import Colors
from picsellia.decorators import exception_handler
from picsellia.sdk.connexion import Connexion
from picsellia.sdk.dao import Dao
from picsellia.sdk.model_version import ModelVersion
from picsellia.sdk.tag import Tag, TagTarget
from picsellia.sdk.taggable import Taggable
from picsellia.types.enums import Framework, InferenceType
from picsellia.types.schemas import ModelSchema

logger = logging.getLogger("picsellia")


class Model(Dao, Taggable):
    def __init__(self, connexion: Connexion, data: dict):
        Dao.__init__(self, connexion, data)
        Taggable.__init__(self, TagTarget.MODEL)

    @property
    def name(self) -> str:
        """Name of this (Model)"""
        return self._name

    @property
    def type(self) -> InferenceType:
        """Type of this (Model)"""
        return self._type

    @property
    def framework(self) -> Framework:
        """Framework of this (Model)"""
        return self._framework

    @property
    def private(self) -> bool:
        """Privacy of this (Model)"""
        return self._private

    def __str__(self):
        is_private = "" if self._private else "[PUBLIC]"
        return f"{Colors.BLUE}{is_private} Model '{self.name}' with type {self.type.name} and framework {self.framework.name}  {Colors.ENDC} (id: {self.id})"

    @exception_handler
    @beartype
    def get_resource_url_on_platform(self) -> str:
        """Get platform url of this resource.

        Examples:
            ```python
            print(foo_dataset.get_resource_url_on_platform())
            >>> "https://app.picsellia.com/model/62cffb84-b92c-450c-bc37-8c4dd4d0f590"
            ```

        Returns:
            Url on Platform for this resource
        """

        return f"{self.connexion.host}/model/{self.id}"

    @exception_handler
    @beartype
    def sync(self) -> dict:
        r = self.connexion.get(f"/sdk/model/{self.id}").json()
        self.refresh(r)
        return r

    @exception_handler
    @beartype
    def refresh(self, data: dict):
        schema = ModelSchema(**data)
        self._name = schema.name
        self._type = schema.type
        self._framework = schema.framework
        self._private = schema.private
        return schema

    @exception_handler
    @beartype
    def update(
        self,
        name: Optional[str] = None,
        framework: Union[str, Framework, None] = None,
        private: Optional[bool] = None,
        description: Optional[str] = None,
        type: Union[str, InferenceType, None] = None,
    ) -> None:
        """Update a model with a new name, framework, privacy, description or type

        Examples:
            ```python
            model.update(description="Very cool model")
            ```
        """
        if framework:
            framework = Framework.validate(framework)

        if type:
            type = InferenceType.validate(type)

        if type and type not in [
            InferenceType.CLASSIFICATION,
            InferenceType.OBJECT_DETECTION,
            InferenceType.SEGMENTATION,
        ]:
            raise TypeError(f"Type '{type}' not supported yet for model")

        payload = {
            "name": name,
            "type": type,
            "framework": framework,
            "private": private,
            "description": description,
        }
        filtered_payload = utils.filter_payload(payload)
        r = self.connexion.patch(
            f"/sdk/model/{self.id}", data=orjson.dumps(filtered_payload)
        ).json()
        self.refresh(r)
        logger.info(f"{self} updated.")

    @exception_handler
    @beartype
    def delete(self) -> None:
        """Delete model.

        Delete the model in Picsellia database

        Examples:
            ```python
            model.delete()
            ```
        """
        self.connexion.delete(f"/sdk/model/{self.id}")
        logger.info(f"{self} deleted.")

    @exception_handler
    @beartype
    def get_tags(self) -> List[Tag]:
        """Retrieve the tags of your model.

        Examples:
            ```python
            tags = my_model.get_tags()
            assert tags[0].name == "my-model-1"
            ```

        Returns:
            List of tags as Tag
        """
        r = self.sync()
        return list(map(partial(Tag, self.connexion), r["tags"]))

    @exception_handler
    @beartype
    def create_version(
        self,
        docker_image_name: Optional[str] = None,
        docker_flags: Optional[List[str]] = None,
        thumb_object_name: Optional[str] = None,
        notebook_link: Optional[str] = None,
        labels: Optional[dict] = None,
        base_parameters: Optional[dict] = None,
        docker_env_variables: Optional[dict] = None,
    ) -> ModelVersion:
        """Create a version of a model.

        The version number of this model will be defined by the platform. It is incremented automatically.

        Examples:
            ```python
            model_v0 = model.create_version()
            ```

        Returns:
            A (ModelVersion) object
        """
        payload = {
            "docker_image_name": docker_image_name,
            "docker_flags": docker_flags,
            "thumb_object_name": thumb_object_name,
            "notebook_link": notebook_link,
            "labels": labels,
            "base_parameters": base_parameters,
            "docker_env_variables": docker_env_variables,
        }
        filtered_payload = utils.filter_payload(payload)
        r = self.connexion.post(
            f"/sdk/model/{self.id}/versions",
            data=orjson.dumps(filtered_payload),
        ).json()
        return ModelVersion(self.connexion, r)

    @exception_handler
    @beartype
    def get_version(self, version: int) -> ModelVersion:
        """Retrieve a version of a model

        Examples:
            ```python
            version = model.get_version(0)
            ```

        Returns:
            A (ModelVersion) object
        """
        params = {"version": version}
        r = self.connexion.get(
            f"/sdk/model/{self.id}/versions/find", params=params
        ).json()
        return ModelVersion(self.connexion, r)

    @exception_handler
    @beartype
    def list_versions(
        self,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        order_by: Optional[List[str]] = None,
    ) -> List[ModelVersion]:
        """List versions of a model

        Examples:
            ```python
            versions = model.list_versions()
            ```

        Returns:
            A list of (ModelVersion) object of this model
        """
        params = {"limit": limit, "offset": offset, "order_by": order_by}
        params = utils.filter_payload(params)
        r = self.connexion.get(f"/sdk/model/{self.id}/versions", params=params).json()
        return list(map(partial(ModelVersion, self.connexion), r["items"]))



import logging
import os
import warnings
from pathlib import Path
from typing import Optional, Union

import orjson
from beartype import beartype
from beartype.roar import BeartypeDecorHintPep585DeprecationWarning

from picsellia.colors import Colors
from picsellia.decorators import exception_handler
from picsellia.sdk.connexion import Connexion
from picsellia.sdk.dao import Dao
from picsellia.types.schemas import ArtifactSchema
from picsellia.utils import filter_payload

logger = logging.getLogger("picsellia")
warnings.filterwarnings("ignore", category=BeartypeDecorHintPep585DeprecationWarning)


class Artifact(Dao):
    def __init__(self, connexion: Connexion, data: dict):
        Dao.__init__(self, connexion, data)

    def __str__(self):
        return f"{Colors.GREEN}Artifact {self.name}{Colors.ENDC} (id: {self.id})"

    @property
    def filename(self) -> str:
        """Filename of this (Artifact)"""
        return self._filename

    @property
    def large(self) -> bool:
        """If true, this (Artifact) has a large size"""
        return self._large

    @property
    def name(self) -> str:
        """(Artifact) name"""
        return self._name

    @property
    def object_name(self) -> str:
        """(Artifact) object name stored in storage"""
        return self._object_name

    @exception_handler
    @beartype
    def refresh(self, data: dict) -> ArtifactSchema:
        schema = ArtifactSchema(**data)
        self._name = schema.name
        self._object_name = schema.object_name
        self._large = schema.large
        self._filename = schema.filename
        return schema

    @exception_handler
    @beartype
    def sync(self) -> dict:
        r = self.connexion.get(f"/sdk/artifact/{self.id}").json()
        self.refresh(r)
        return r

    @exception_handler
    @beartype
    def update(
        self,
        name: Optional[str] = None,
        filename: Optional[str] = None,
        object_name: Optional[str] = None,
        large: Optional[bool] = None,
    ) -> None:
        """Update this artifact.

        Examples:
            ```python
            this_artifact.update(object_name="another-path-to-artifact")
            ```
        """
        payload = {
            "name": name,
            "filename": filename,
            "object_name": object_name,
            "large": large,
        }
        filtered_payload = filter_payload(payload)
        r = self.connexion.patch(
            f"/sdk/artifact/{self.id}", data=orjson.dumps(filtered_payload)
        ).json()
        self.refresh(r)

    @exception_handler
    @beartype
    def delete(self) -> None:
        """Delete this artifact

        Examples:
            ```python
            this_artifact.delete()
            ```
        """
        self.connexion.delete(f"/sdk/artifact/{self.id}")

    @exception_handler
    @beartype
    def download(
        self, target_path: Union[str, Path] = "./", force_replace: bool = False
    ) -> None:
        """Download an experiment's artifact to a given target_path.

        Examples:
            ```python
            this_artifact.download("myDir")
            file_list = os.path.listdir("myDir")
            print(file_list)
            >>> ["saved_model.zip"]
            ```
        Arguments:
            target_path (str or Path, optional): Path to download the file to, default to cwd. Defaults to './'.
            force_replace: (bool, optional): Replace an existing file if exists. Defaults to False.

        """
        self.sync()
        path = os.path.join(target_path, self.filename)
        if self.connexion.download_file(
            self.object_name, path, self.large, force_replace=force_replace
        ):
            logger.info(f"{self.filename} downloaded successfully")
        else:  # pragma: no cover
            logger.error(f"Did not download file '{self.name}' ({self.filename})")



import logging
import os
import warnings
from pathlib import Path
from typing import Optional, Union

import orjson
from beartype import beartype
from beartype.roar import BeartypeDecorHintPep585DeprecationWarning

from picsellia.colors import Colors
from picsellia.decorators import exception_handler
from picsellia.sdk.connexion import Connexion
from picsellia.sdk.dao import Dao
from picsellia.types.schemas import LoggingFileSchema
from picsellia.utils import filter_payload

logger = logging.getLogger("picsellia")
warnings.filterwarnings("ignore", category=BeartypeDecorHintPep585DeprecationWarning)


class LoggingFile(Dao):
    def __init__(self, connexion: Connexion, data: dict):
        Dao.__init__(self, connexion, data)
        self.text = ""

    def __str__(self):
        return f"{Colors.GREEN}Logging file {self.object_name}{Colors.ENDC} (id: {self.id})"

    @property
    def object_name(self) -> str:
        """Object name of this (LoggingFile)"""
        return self._object_name

    @exception_handler
    @beartype
    def refresh(self, data: dict) -> LoggingFileSchema:
        schema = LoggingFileSchema(**data)
        self._object_name = schema.object_name
        return schema

    @exception_handler
    @beartype
    def sync(self) -> dict:
        r = self.connexion.get(f"/sdk/logging/file/{self.id}").json()
        self.refresh(r)
        return r

    @exception_handler
    @beartype
    def update(
        self,
        object_name: Optional[str] = None,
    ) -> None:
        """Update this artifact.

        Examples:
            ```python
            this_artifact.update(object_name="another-path-to-artifact")
            ```
        """
        payload = {
            "object_name": object_name,
        }
        filtered_payload = filter_payload(payload)
        r = self.connexion.patch(
            f"/sdk/logging/file/{self.id}", data=orjson.dumps(filtered_payload)
        ).json()
        self.refresh(r)

    @exception_handler
    @beartype
    def delete(self) -> None:
        """Delete this artifact

        Examples:
            ```python
            this_artifact.delete()
            ```
        """
        self.connexion.delete(f"/sdk/logging/file/{self.id}")

    @exception_handler
    @beartype
    def download(
        self, target_path: Union[str, Path] = "./", force_replace: bool = False
    ) -> None:
        """Download an experiment logging file to a given target_path.

        Examples:
            ```python
            logging_file.download("myDir")
            file_list = os.path.listdir("myDir")
            print(file_list)
            >>> ["saved_model.zip"]
            ```
        Arguments:
            force_replace: (bool, optional): If true, force replacement of existing disk file. Defaults to false.
            target_path (str or Path, optional): Path to download the file to, default to cwd. Defaults to './'.

        """
        self.sync()
        filename = self.object_name.split("/")[-1]
        path = os.path.join(target_path, filename)
        if self.connexion.download_file(
            self.object_name, path, False, force_replace=force_replace
        ):
            logger.info(f"{filename} downloaded successfully")
        else:  # pragma: no cover
            logger.error(f"Did not download logging file of {self}")
